---
title: Towards computer assisted modeling
theme: beige
revealOptions:
    transition: 'slide'
    slideNumber: true
---

<img style="height:350px" src=./images/openmole5.png>

## Model Evaluation with OpenMOLE

#### Jonathan Passerat-Palmbach, Romain Reuillon

---

## OpenMOLE Manifesto

 * Provide state-of-the-art exploration methods
 * Tackle scalability using distributed computing
 * Is not intrusive in the model logic

---

<img src=./images/openmoleGalRaw.png>

---

## 1 - Model?
<img src="./images/modelIO.png"/> <br/><br/>

----

<img src=./images/blackbox.png>

----

<img src=./images/openmolefirefox.png>

----

## NetLogo on Fire

<pre style="brush: scala"><code>
val model =
  NetLogo6Task(
    workDirectory / "Fire.nlogo",
    List("setup", "while [any? turtles] [go]")) set (
    inputs += seed,
    outputs += (seed, density),
    netLogoInputs += (density, "density"),
    netLogoOutputs += ("burned-trees", burned)
  )
</code></pre>

----

## Rrrrrrrrrr

<pre style="brush: scala"><code>
val i = Val[Int]

val rTask =
  RTask("""
    source("function.R")
    function(i)""") set (
    resources += workDirectory / "function.R",
    rInputs += i
  )
</code></pre>


----

## Feeling snaky? Python

### Package
<pre><code>care -o python.tgz.bin python matrix.py data.csv 1 out.csv</code></pre>

### Run
<pre style="brush: scala"><code>
val pyTask =
  CARETask(
    workDirectory / "../python.tgz.bin",
    "python matrix.py data.csv 2 out.csv"
  )
</code></pre>


---

## 2 - Method?
      
<img src="./images/openmoleGalRaw.png"><br>

----

<ul>
  <li>Data reconstruction</li>
  <li>Parameter estimation</li>
  <li>Sensitivity analysis</li>
  <li>Robustness assessment</li>
  <li>Optimisation</li>
  <li>...</li>
</ul>


----

<div style="margin-top:40px">Designed in a <b>scalable</b> manner, handle <b>stochasticity</b>, usable on <b>any models and environments</b>.</div>

----

## Scripting

<pre style="brush: scala"><code>DirectSampling(
  evaluation = myModel,
  sampling =
    LHS(
      500 // number of points,
      diffusion in (10.0, 100.0),
      evaporation in (10.0, 100.0)
    )
)
</code></pre>


----

<img src="./images/three_LHS.png"/>

----

<img src="./images/ill-be-back.jpg">
<div style="align: center; margin-top: 30px">**Let's talk about more methods later**</div>

---

## 3 - Execution environment?

<img src="./images/openmoleGalRaw.png">

----

## Prototype Small, Scale for Free

<img src="./images/scale.jpg"/>

----

## Zero deployment approach
        
* No prior knowledge of remote environment needed
* No installation required on any machine
* User code and data are automatically deployed

----

## Scripting

<pre style="brush: scala"><code>
DirectSampling(
  evaluation = myModel,
  sampling =   
    LHS(
      500,
      diffusion in (10.0, 100.0),
      evaporation in (10.0, 100.0)
    )
)
</code></pre>

----

## Scripting

<pre style="brush: scala"><code>
val cluster = SLURMEnvironment("login", "cluster.domain.org")

DirectSampling(
  evaluation = myModel on cluster,
  sampling =   
    LHS(
      500,
      diffusion in (10.0, 100.0),
      evaporation in (10.0, 100.0)
    )
)
</code></pre>

----

## Supported environments today

* Multi-thread
* Delegation through SSH
* PBS
* SLURM
* Condor
* SGE
* OAR
* EGI Grid

---

## I am back!

<img src="./images/terminator2.jpg">

---

## Computer assisted modeling

<p style="text-align:center">Build a general framework and algorithms to guide the modeling process.</p><br/>

Make the modeling process:
  * *traceable*: understand what choices were made,
  * *reproducible*: recompute the reason why they were made,
  * *reusable*: study alternative choices.

----

### Descriptive models

<p style="text-align:center">Design/evaluate a theory involving causal effects through its capacity to (re-)produce some patterns/data.</p><br/>
<img style="height:300px" src=img/descriptive_model.svg>

----

### Model evaluation

How to assess: 
  * the *sufficiency* of the mechanisms?
  * the *necessity* of the mechanisms?
  * the *uniqueness* of the mechanisms?

---

## Sufficiency

----

### Free parameters, example of Simpop Local

<img style="height:400px" src=img/slocal.png>

<p style="text-align:center">No empirical evidence for the parameters.</p>

----

### The usual approach

<p style="text-align:center">Use a classical design of experiment.</b> <br/>


<img style="height:400px" src=img/sobol.jpg>

----

### Limits

 * It produces a huge quantity of data.
 * It transforms a problem into a data-mining problem.
 * The parameter space is mostly left unexplored due to the curse of dimensionality.

----

### Reverse approach

<img style="height:400px" src=img/guided.jpg>

<p style="text-align:center">From *output* values to *parameter* values.</b>

----

### Formalising the expectations

<img style="height:500px" src=img/slocal_fitness.png>

----

### Genetic algorithm as a calibration algorithm

<img style="height:500px" src=img/ga.png>

----

### Calibration method

*Genome*: 6 parameters of the model

*Fitness*: 3 descriptors of the quality of the simulated dynamic.

<br/>

Necessary extensions:
  * distributed computing
  * auto-adaptative strategy to handle stochasticity

----

### Results

<img style="height:450px" src=img/simpopcalibrated.svg>

<p style="text-align:center">No compromise between the 3 objectives.</p>

----

### Evaluation approach

* Would not have been found using a direct method.

<img style="height:300px" src=img/hv.svg>

The method is tractable (even for ABMs):
  * Handles stochasticity: 100x gain.
  * Support for distributed computing: 1000x gain.

----

### Conclusion

How to assess: 
  * the sufficiency of the mechanisms ✓
  * the necessity of the mechanisms ?
  * the uniqueness of the mechanisms ?

---

## Necessity

----

### A new algorithm

* To detect if a parameter is useful: it impacts the capacity of the model to produce plausible outcomes.

* To better constrain the parameter ranges.

* As an indirect way to detect if some of the mechanisms are expandable. 


----

### Objective

<p style="text-align:center">Hundreds of calibrations</p>

<img style="height:450px" src=img/PdiffusionZ.pdf.png>

----

<img style="height:500px" src=img/profil_algo.svg>

----

### The Profiles algorithm

Compute the best calibration for hundreds of values along the definition domain of a parameter.

<br/>

The method is tractable (even for ABMs):
  * Converges almost as fast as a single calibration.
  * Handles stochasticity: 100x gain.
  * Support for distributed computing: 1000x gain.

----

### Interpretation


<img style="height:500px" src=img/profil_interpretation.svg>

----

### Results

<img style="height:500px" src=img/PdiffusionZ.pdf.png>


----

### Results

<img style="height:500px" src=img/InnovationLife.pdf.png>

----

### Conclusion

How to assess: 
  * the sufficiency of the mechanisms ✓
  * the necessity of the mechanisms ✓
  * the uniqueness of the mechanisms ?

---

## Uniqueness

----

<img style="height:300px" src=img/descriptive_model.svg>

<br/>
<p style="text-align:center">Automate the confrontation of alternative hypotheses / mechanisms.</p>

----

<img style="height:400px" src=img/model_familly.svg>

----

### What are we running after?

<img style="height:400px" src=img/familly_pareto.svg>

----

<img style="height:450px" src=img/marius_complexification.png>

<br/>

<p style="text-align:center">64 models</p>

----

### Example of concurrent hypotheses

<div id="left" style="width:40%"><img style="height:350px;padding-top:50px" src="./img/system_mec.png"></div>

<div id="right" style="width:55%">
<p>Exchange mechanism:</p>
<ul>
 <li>Market based</li>
 <li>Centrally planned</li>
</ul>
<br/>
<p>City growth mechanism:</p>
<ul>
 <li>Purely inter-urban interactions</li>
 <li>Influenced by environmental situation</li>
</ul>
</div>

----

### Model familly calibration

Compute the best set of parameters for all 64 models.

<img style="height:400px" src=img/varius_big.png>

[http://shiny.parisgeo.cnrs.fr/VARIUS/](http://shiny.parisgeo.cnrs.fr/VARIUS/)

----

### Calibration method

*Algorithm principle*: reuse the good parameters found on one model to calibrate on the others.

<br/>

The method is tractable (even for ABMs):
  * as fast (faster ??) as calibrating a single model.
  * Handles stochasticity: 100x gain.
  * Support for distributed computing: 1000x gain.

----

### Conclusion

How to assess: 
  * the sufficiency of the mechanisms ✓
  * the necessity of the mechanisms ✓
  * the uniqueness of the mechanisms (≈) ✓

----

### Alternative aproach

<p style="text-align:center">All these methods are based on an *a priori* mesurment of the quality.</p>

<img style="height:300px" src=img/narowed.jpg>

<br/>

<p style="text-align:center">Can we propose an alternative aproach to evaluation?</p>

---

## Pattern search

----

<img style="height:500px" src=img/pse_algo.svg>

----

<img style="height:400px" src=img/hitmap.svg>

<p style="text-align:center;padding-top:10px">The inputs producing rare patterns have high fitness values.</p>

----

<img style="height:500px" src=img/flockingvolumediscovered.svg>

----

<img style="height:500px" src=img/flockingpatterns.png>

----

<img style="height:600px" src=img/pse_marius.png>

----

### Interpretation

* All expected/observed patterns are produced: the model is generic enough.<br/><br/>

* Unexpected patterns are found:
 * aberrations: better constrain the model,
 * predictions: test them.

----

### Exemple OpenMOLE + SimPLU (IGN)

<video height="400px" autoplay controls>
  <source src="img/capture_visuPSE.webm" type="video/webm">
</video>

[simplu.openmole.org](http://simplu.openmole.org)

---

## Origin search

----

<img style="height:500px" src=img/ose_algo.svg>

----

<p style="text-align:center">Formulation: Δ pattern < ε</p> 

<img style="height:400px" src=img/rastrigin.svg>

<p style="text-align:center">f(x1, x2, .., xn) < 10</p>

----

<p style="text-align:center">6-D Rastrigin function < 10</p>

<img style="height:400px" src=img/oserast6d.png>


---
## Bye, bye!

<img src="./images/terminator2.jpg">

---

## Want to use it?

* *Stable*: <a href="http://openmole.org" style="display:inline; font-size: 100%">openmole.org</a>
* *Dev*: <a href="http://next.openmole.org" style="display:inline; font-size: 100%">next.openmole.org</a>
* *User forum*: <a href="http://ask.openmole.org" style="display:inline; font-size: 100%">ask.openmole.org</a>
* *Source code*: <a href="http://github.com/openmole/openmole" style="display:inline; font-size: 100%">github.com/openmole/openmole</a>
* *Demo instance*: <a href="http://demo.openmole.org" style="display:inline; font-size: 100%">demo.openmole.org</a>

---

## ISC-PIF services

* *Grid accesss*: <a href="http://iscpif.fr/services" style="display:inline; font-size: 100%">iscpif.fr/services</a>
* *Discussion / Diffusion* maling list: <a href="http://discuss.iscpif.fr" style="display:inline; font-size: 100%">discuss.iscpif.fr</a>

